-- if we're in CourseMode, bail now
-- the normal LifeMeter graph (Def.GraphDisplay) will be drawn
if GAMESTATE:IsCourseMode() then return end

-- arguments passed in from Graphs.lua
local args = ...
local player = args.player
local GraphWidth = args.GraphWidth
local GraphHeight = args.GraphHeight

-- detailed_judgments gathered in ./BGAnimations/ScreenGameplay overlay/DetailedJudgmentTracking.lua
local detailed_judgments = SL[ToEnumShortString(player)].Stages.Stats[SL.Global.Stages.PlayedThisGame + 1].detailed_judgments

-- a table to store the AMV's vertices
local verts= {}
-- TotalSeconds is used in scaling the x-coordinates of the AMV's vertices
local FirstSecond = GAMESTATE:GetCurrentSong():GetFirstSecond()
local TotalSeconds = GAMESTATE:GetCurrentSong():GetLastSecond()

-- variables that will be used and re-used in the loop while calculating the AMV's vertices
local Offset, CurrentSecond, TimingWindow, x, y, c, r, g, b

-- ---------------------------------------------
-- if players have disabled W4 or W4+W5, there will be a smaller pool
-- of judgments that could have possibly been earned
local worst_window = PREFSMAN:GetPreference("TimingWindowSecondsW5")
local windows = SL.Global.ActiveModifiers.TimingWindows
for i=5,1,-1 do
	if windows[i] then
		worst_window = PREFSMAN:GetPreference("TimingWindowSecondsW"..i)
		break
	end
end

-- ---------------------------------------------

local colors = {}
for w=5,1,-1 do
	if SL.Global.ActiveModifiers.TimingWindows[w]==true then
		colors[w] = DeepCopy(SL.JudgmentColors.Waterfall[w])
	else
		colors[w] = DeepCopy(colors[w+1] or SL.JudgmentColors.Waterfall[w+1])
	end
end

-- ---------------------------------------------

for t in ivalues(detailed_judgments) do
	CurrentSecond = t[1]
	TimingWindow = t[2]
	if (TimingWindow ~= "Held" and TimingWindow ~= "LetGo") then
		Offset = (t[2] ~= "HitMine") and t[4] or 0

		if TimingWindow ~= "Miss" then
			CurrentSecond = CurrentSecond - Offset
		else
			CurrentSecond = CurrentSecond - worst_window
		end

		-- pad the right end because the time measured seems to lag a little...
		x = scale(CurrentSecond, FirstSecond, TotalSeconds + 0.05, 0, GraphWidth)

		if TimingWindow ~= "Miss" and TimingWindow ~= "HitMine" then
			local wid = tonumber(TimingWindow:sub(-1))
			y = scale(Offset, worst_window, -worst_window, 0, GraphHeight)

			-- get the appropriate color from the global SL table
			c = colors[wid]
			-- get the red, green, and blue values from that color
			r = c[1]
			g = c[2]
			b = c[3]

			-- insert four datapoints into the verts tables, effectively generating a single quadrilateral
			-- top left,  top right,  bottom right,  bottom left
			table.insert( verts, {{x,y,0}, {r,g,b,0.666}} )
			table.insert( verts, {{x+1.5,y,0}, {r,g,b,0.666}} )
			table.insert( verts, {{x+1.5,y+1.5,0}, {r,g,b,0.666}} )
			table.insert( verts, {{x,y+1.5,0}, {r,g,b,0.666}} )
		else
			-- else, a miss should be a quadrilateral that is the height of the entire graph and red
			-- similarly for a mine, but make mines gray
			local clr = TimingWindow == "Miss" and color("#ff000077") or color("#82828277")
			table.insert( verts, {{x, 0, 0}, clr} )
			table.insert( verts, {{x+1, 0, 0}, clr} )
			table.insert( verts, {{x+1, GraphHeight, 0}, clr} )
			table.insert( verts, {{x, GraphHeight, 0}, clr} )
		end
	end
end

-- the scatter plot will use an ActorMultiVertex in "Quads" mode
-- this is more efficient than drawing n Def.Quads (one for each judgment)
-- because the entire AMV will be a single Actor rather than n Actors with n unique Draw() calls.
local amv = Def.ActorMultiVertex{
	InitCommand=function(self) self:x(-GraphWidth/2) end,
	OnCommand=function(self)
		self:SetDrawState({Mode="DrawMode_Quads"})
			:SetVertices(verts)
	end,
}

return amv
